from typing import Dict, List, Optional, Union

import infima_client.api.prediction_service.prediction_service_get_latest_cohort as prediction_service_get_latest_cohort
import infima_client.api.prediction_service.prediction_service_get_latest_pools as prediction_service_get_latest_pools
import infima_client.api.prediction_service.prediction_service_get_latest_portfolio as prediction_service_get_latest_portfolio
from infima_client.models import *

from .client import Client
from .types import UNSET, Response, Unset


class prediction:
    def __init__(self, client: Client):
        self.client = client
        self._detailed = predictionDetailed(client)

    def get_latest_cohort(
        self, cohort: str
    ) -> Optional[
        Union[
            LatestCohortPrediction,
            LatestCohortPredictionBadRequest,
            LatestCohortPredictionUnauthorized,
            LatestCohortPredictionNotFound,
            LatestCohortPredictionError,
        ]
    ]:

        """
        Fetch most recent prepayment prediction for a single cohort.

        Args:
            cohort: Cohort name to fetch aggregated prepayment prediction.

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return prediction_service_get_latest_cohort.sync(
            client=self.client, cohort=cohort
        )

    def get_latest_pools(
        self, cusips: List[str]
    ) -> Optional[
        Union[
            LatestPoolPredictions,
            LatestPoolPredictionsBadRequest,
            LatestPoolPredictionsUnauthorized,
            LatestPoolPredictionsNotFound,
            LatestPoolPredictionsError,
        ]
    ]:

        """
        Fetch most recent individual pool prepayment predictions.

        Args:
            cusips: Pool CUSIP or CUSIPS to fetch individual prepayment predictions.

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return prediction_service_get_latest_pools.sync(
            client=self.client, cusips=cusips
        )

    def get_latest_portfolio(
        self, cusips: List[str]
    ) -> Optional[
        Union[
            LatestPortfolioPrediction,
            LatestPortfolioPredictionBadRequest,
            LatestPortfolioPredictionUnauthorized,
            LatestPortfolioPredictionNotFound,
            LatestPortfolioPredictionError,
        ]
    ]:

        """
        Weighted aggregate of the prepayment prediction of the portfolio components.

        Args:
            cusips: Pool CUSIPS to fetch aggregated prepayment prediction.

        Returns:
            Either a valid response or an unauthorized, not found or other error.
        """

        return prediction_service_get_latest_portfolio.sync(
            client=self.client, cusips=cusips
        )


class predictionDetailed:
    def __init__(self, client: Client):
        self.client = client

    def get_latest_cohort(
        self, cohort: str
    ) -> Response[
        Union[
            LatestCohortPrediction,
            LatestCohortPredictionBadRequest,
            LatestCohortPredictionUnauthorized,
            LatestCohortPredictionNotFound,
            LatestCohortPredictionError,
        ]
    ]:

        return prediction_service_get_latest_cohort.sync_detailed(
            client=self.client, cohort=cohort
        )

    def get_latest_pools(
        self, cusips: List[str]
    ) -> Response[
        Union[
            LatestPoolPredictions,
            LatestPoolPredictionsBadRequest,
            LatestPoolPredictionsUnauthorized,
            LatestPoolPredictionsNotFound,
            LatestPoolPredictionsError,
        ]
    ]:

        return prediction_service_get_latest_pools.sync_detailed(
            client=self.client, cusips=cusips
        )

    def get_latest_portfolio(
        self, cusips: List[str]
    ) -> Response[
        Union[
            LatestPortfolioPrediction,
            LatestPortfolioPredictionBadRequest,
            LatestPortfolioPredictionUnauthorized,
            LatestPortfolioPredictionNotFound,
            LatestPortfolioPredictionError,
        ]
    ]:

        return prediction_service_get_latest_portfolio.sync_detailed(
            client=self.client, cusips=cusips
        )
